var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { makeKMap, SyncModels, } from '..';
import { sleep } from '../helpers/sleep.func';
export class BaseCloudSynchronizer extends SyncModels.keyValue {
    constructor(cloudSyncOptions) {
        var _a;
        super(Object.assign({ name: '_cloudSynchronizer' }, ((_a = cloudSyncOptions.mixinOptions) !== null && _a !== void 0 ? _a : {})));
        this.models = makeKMap();
        this.pushDebounceTimer = 'idle';
        this.firstSync = true;
        if (!cloudSyncOptions.driver)
            throw new Error(`[Kurimudb] To use the cloud synchronization function, you must pass in the "driver".`);
        this.cloudSyncStorage = cloudSyncOptions.driver.make(this);
        if (undefined === cloudSyncOptions.autoPush)
            cloudSyncOptions.autoPush = true;
        if (undefined === cloudSyncOptions.disabled)
            cloudSyncOptions.disabled = false;
        if (undefined === cloudSyncOptions.minSyncInterval)
            cloudSyncOptions.minSyncInterval = 2000;
        if (undefined === cloudSyncOptions.retryDelayInterval)
            cloudSyncOptions.retryDelayInterval = 1000;
        if (undefined === cloudSyncOptions.maxRetries)
            cloudSyncOptions.maxRetries = 3;
        this.cloudSyncOptions = cloudSyncOptions;
        const setResult = (type, modelName, key, value) => __awaiter(this, void 0, void 0, function* () {
            const timestampMs = new Date().getTime();
            yield this._pushWaitingPushData([
                {
                    type: type,
                    modelName: modelName,
                    timestampMs: timestampMs,
                    key: key,
                    value: value,
                    pushing: false,
                },
            ]);
            yield this.storageSetItem(`_keyUpdateTimestampMs-${modelName}-${key}`, timestampMs);
            if (true === this.cloudSyncOptions.autoPush) {
                this.push(true);
            }
        });
        for (const model of cloudSyncOptions.models) {
            this.models[model.options.name] = model;
            model.setItem$((value, key) => __awaiter(this, void 0, void 0, function* () {
                yield setResult('put', model.options.name, key, value);
            }));
            model.removeItem$((key) => __awaiter(this, void 0, void 0, function* () {
                yield setResult('forget', model.options.name, key, null);
            }));
            if ('insertItem$' in model) {
                model.insertItem$((value, key) => __awaiter(this, void 0, void 0, function* () {
                    yield setResult('put', model.options.name, key, value);
                }));
            }
        }
    }
    pull() {
        return __awaiter(this, void 0, void 0, function* () {
            const _currentPullTimestampMs = new Date().getTime();
            let _lastPullTimestampMs = yield this.storageGetItem('_lastPullTimestampMs');
            if (undefined === _lastPullTimestampMs)
                _lastPullTimestampMs = 0;
            let results;
            try {
                results = yield this.cloudSyncOptions.pullDataHandler(_lastPullTimestampMs);
            }
            catch (error) {
                throw error;
            }
            for (const i in results) {
                const item = results[i];
                let localKeyUpdateTimestampMs = (yield this.storageGetItem(`_keyUpdateTimestampMs-${item.modelName}-${item.key}`));
                if (undefined === localKeyUpdateTimestampMs) {
                    localKeyUpdateTimestampMs = 0;
                }
                if (item.timestampMs > localKeyUpdateTimestampMs) {
                    const model = this.models[item.modelName];
                    yield model.setItem(item.key, item.value);
                    yield this.storageSetItem(`_keyUpdateTimestampMs-${item.modelName}-${item.key}`, item.timestampMs);
                }
                else {
                    continue;
                }
            }
            yield this.storageSetItem('_lastPullTimestampMs', _currentPullTimestampMs);
            yield sleep(42);
        });
    }
    push(passive = false) {
        let timeout;
        if ('idle' === this.pushDebounceTimer) {
            if (false === passive) {
                timeout = 0;
            }
            else {
                timeout = this.cloudSyncOptions.minSyncInterval;
            }
        }
        else if ('busy' === this.pushDebounceTimer) {
            timeout = this.cloudSyncOptions.minSyncInterval;
        }
        else {
            clearTimeout(this.pushDebounceTimer);
            timeout = this.cloudSyncOptions.minSyncInterval;
        }
        const setPushDebounceTimer = () => {
            this.pushDebounceTimer = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                if ('busy' === this.pushDebounceTimer)
                    return setPushDebounceTimer();
                this.pushDebounceTimer = 'busy';
                this.setItem('syncStatus', 'pending');
                yield this._recursionPush(undefined, undefined);
                this.pushDebounceTimer = 'idle';
            }), timeout);
        };
        setPushDebounceTimer();
    }
    _pushData(waitingPushData) {
        return __awaiter(this, void 0, void 0, function* () {
            waitingPushData = waitingPushData.sort((a, b) => a.timestampMs - b.timestampMs);
            let newWaitingPushData = makeKMap();
            for (const i in waitingPushData) {
                delete waitingPushData[i].pushing; // This is done to convert WaitingPushData to PushData
                newWaitingPushData[`${waitingPushData[i].timestampMs}-${waitingPushData[i].modelName}-${waitingPushData[i].key}-${waitingPushData[i].type}`] = waitingPushData[i];
            }
            return yield this.cloudSyncOptions.pushDataHandler(newWaitingPushData.toArray());
        });
    }
    _recursionPush(_waitingPushData, maxRetries) {
        return __awaiter(this, void 0, void 0, function* () {
            if (undefined === maxRetries) {
                maxRetries = this.cloudSyncOptions.maxRetries;
            }
            if (undefined === _waitingPushData) {
                _waitingPushData = yield this._processWaitingPushData();
            }
            if (0 === _waitingPushData.length)
                return;
            if (false === (yield this._pushData(_waitingPushData))) {
                yield sleep(this.cloudSyncOptions.retryDelayInterval);
                --maxRetries;
                if (0 >= maxRetries) {
                    // maxRetries is exhausted, defeated.
                    yield this._processedWaitingPushData(_waitingPushData, false);
                    this.setItem('syncStatus', 'defeated');
                    return;
                }
                // retry recursively.
                return yield this._recursionPush(_waitingPushData, maxRetries);
            }
            else {
                // success!!
                yield this._processedWaitingPushData(_waitingPushData, true);
                this.setItem('syncStatus', 'succeed');
                return;
            }
        });
    }
    _pushWaitingPushData(item) {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.storageGetItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            _waitingPushData.push(...item);
            this.storageSetItem('_waitingPushData', _waitingPushData);
        });
    }
    _processWaitingPushData() {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.storageGetItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            let newWaitingPushData = [];
            _waitingPushData.forEach((item) => {
                if (true === this.firstSync && true === item.pushing)
                    return;
                item.pushing = true;
                newWaitingPushData.push(Object.assign({}, item));
            });
            this.firstSync = false;
            this.storageSetItem('_waitingPushData', _waitingPushData);
            return newWaitingPushData;
        });
    }
    _processedWaitingPushData(waitingPushData, isSuccess) {
        return __awaiter(this, void 0, void 0, function* () {
            let _waitingPushData = yield this.storageGetItem('_waitingPushData');
            if (undefined === _waitingPushData || null === _waitingPushData) {
                _waitingPushData = [];
            }
            console.log('hello');
            for (const i in _waitingPushData) {
                for (const j in waitingPushData) {
                    // console.log(JSON.stringify(_waitingPushData));
                    // console.log(_waitingPushData[i]);
                    // console.log('timestampMs' in _waitingPushData[i]);
                    if (_waitingPushData[i].timestampMs !== waitingPushData[j].timestampMs)
                        continue;
                    if (_waitingPushData[i].modelName !== waitingPushData[j].modelName)
                        continue;
                    if (_waitingPushData[i].type !== waitingPushData[j].type)
                        continue;
                    if (_waitingPushData[i].key !== waitingPushData[j].key)
                        continue;
                    if (true !== _waitingPushData[i].pushing)
                        continue;
                    if (true === isSuccess)
                        delete _waitingPushData[i];
                    else
                        _waitingPushData[i].pushing = false;
                }
            }
            this.storageSetItem('_waitingPushData', _waitingPushData);
        });
    }
    storageGetItem(key) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = this.getItem(key);
            if (undefined === data) {
                data = yield this.cloudSyncStorage.select(key);
            }
            return data;
        });
    }
    storageSetItem(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            this.setItem(key, value);
            yield this.cloudSyncStorage.insertOrUpdate(key, value);
        });
    }
}
//# sourceMappingURL=base-cloud-synchronizer.class.js.map