import { SyncAbstractDriverInterface } from '../../drivers/sync-abstract-driver.class';
import { ModelOptionsInterface } from './model-options.interface';
import { DataProxyType } from '../../data/sync/data-factory.class';
import { SubscribeClosureInterface, SubscribeInterface, UnsubscribeInterface } from '../../cache/subscribe.interface';
import { KMap } from '../../helpers/make-kurimudb-map.func';
import { CacheInterface } from '../../cache/cache-factory.class';
import { SubscribeConfigInterface } from '../../cache/subscribe-config.interface';
interface AutoSubscribeClosureFunctionInterface {
    (): void | Promise<void>;
}
export declare class BaseModel<DataType extends Record<string, any> = Record<string, any>, DriverType extends SyncAbstractDriverInterface | undefined = undefined> {
    readonly options: ModelOptionsInterface;
    readonly cache: CacheInterface;
    readonly $: SubscribeInterface;
    readonly data: DataProxyType<DataType>;
    readonly storage: DriverType;
    constructor(options: Partial<ModelOptionsInterface>);
    private checkOptions;
    _curdSubscribeId: number;
    _curdSubscribeFuncs: {
        setItem: Record<number, Function>;
        removeItem: Record<number, Function>;
    };
    getItem<Key extends keyof DataType>(key: Key): DataType[Key] | undefined;
    bulkGetItem<Key extends keyof DataType>(keys: Array<Key>): KMap<DataType[Key]>;
    setItem<Key extends keyof DataType>(key: Key, value: DataType[Key]): void;
    bulkSetItem<Key extends keyof DataType>(items: Record<Key, DataType[Key]>): boolean;
    setItem$<Key extends keyof DataType>(closFunc: (value: DataType[Key], key: Key) => void): () => boolean;
    hasItem<Key extends keyof DataType>(key: Key): boolean;
    removeItem<Key extends keyof DataType>(key: Key): void;
    bulkRemoveItem<Key extends keyof DataType>(keys: Array<Key>): boolean;
    removeItem$<Key extends keyof DataType>(closFunc: (key: Key) => void): () => boolean;
    subscribeItem<Key extends keyof DataType>(key: Key, closFunc: SubscribeClosureInterface, config?: SubscribeConfigInterface): UnsubscribeInterface;
    getSubscribeItemFunc<Key extends keyof DataType>(key: Key): SubscribeInterface;
    clone<T>(value: T): T;
    auto$(subClosFunc: AutoSubscribeClosureFunctionInterface, config?: SubscribeConfigInterface): UnsubscribeInterface | Promise<UnsubscribeInterface>;
    stream$<Key extends keyof DataType>(getData: () => Array<Key> | Record<Key, DataType[Key]> | Promise<Array<Key> | Record<Key, DataType[Key]>> | Array<string> | Record<string, any> | Promise<Array<string> | Record<Key, any>>, handler: (key: Key, value: DataType[Key]) => void | unknown | Promise<void | unknown>, config?: SubscribeConfigInterface & {
        autoLoad?: boolean;
    }): {
        load: () => Promise<void>;
        startAutoLoad: () => void;
        stopAutoLoad: () => void;
        unsubscribe: () => void;
        getKeys: () => IterableIterator<Key>;
        getLength: () => number;
    };
}
export {};
